using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Cmt6.Exceptions;

namespace Cmt6.Models
{
    public static class EntityExtensions
    {
        public static IEnumerable<PropertyInfo> GetPublicProperties(this IEntity entity)
        {
            return entity.GetType().GetProperties();
        }

        public static IEnumerable<PropertyInfo> GetPublicProperties(this IEntity entity, IEnumerable<string> propertyNames)
        {
            return entity.GetPublicProperties()
                .Where(p => propertyNames.Contains(p.Name, StringComparer.OrdinalIgnoreCase));
        }

        public static PropertyInfo GetPublicProperty(this IEntity entity, string propertyName)
        {
            var property = entity.GetPublicProperties()
                .FirstOrDefault(p => p.Name.Equals(propertyName, StringComparison.OrdinalIgnoreCase));
            if (property == null)
            {
                throw new PropertyNotFoundException(entity.GetType().FullName, propertyName);
            }
            return property;
        }

        public static object GetPublicPropertyValue(this IEntity entity, string propertyName)
        {
            var property = entity.GetPublicProperty(propertyName);
            return entity.GetPublicPropertyValue(property);
        }

        public static object GetPublicPropertyValue(this IEntity entity, PropertyInfo property)
        {
            if (!property.CanRead)
                throw new PropertyNotAccessibleException(entity.GetType().FullName, property.Name, true);
            return property.GetGetMethod().Invoke(entity, null);
        }

        public static object SetPublicPropertyValue(this IEntity entity, string propertyName, object value)
        {
            var property = entity.GetPublicProperty(propertyName);
            if (!property.CanWrite)
                throw new PropertyNotAccessibleException(entity.GetType().FullName, propertyName, false);
            return property.GetSetMethod().Invoke(entity, new[] { value });
        }
    }
}