﻿using System.Web.Mvc;
using Cmt6.Models;
using Cmt6.Models.Mock;
using Cmt6.Repositories;

namespace Cmt6.Presentations.Mvc2.Controllers
{
    public class EntityControllerBase<TEntity> : Controller
        where TEntity : IEntity
    {
        public IEntityRepository<TEntity> EntityRepository { get; set; }
        public IEntityFactory EntityFactory { get; set; }
        public string IndexViewName { get; set; }
        public string EditViewName { get; set; }

        public virtual ViewResult Index()
        {
            ViewData.Add("entityType", typeof(TEntity));
            return View(IndexViewName, EntityRepository.FindAll());
        }

        [HttpGet]
        public virtual ViewResult Create()
        {
            return View(EditViewName, EntityFactory.Create(typeof(TEntity)));
        }
        
        [HttpPost]
        public ActionResult Create(TEntity entity)
        {
            return Edit(entity);
        }

        [HttpGet]
        public virtual ViewResult Edit(int id)
        {
            var entity = EntityRepository.FindById(id);
            return View(EditViewName, entity);
        }

        [HttpPost]
        public ActionResult Edit(TEntity entity)
        {
            //asp.net mvc bug: can not retrive value from field which name is "Id"
            entity.Id = int.Parse(Request.Form["Id"]);
            EntityRepository.AddOrUpdate(entity);

            return RedirectToAction("Index");
        }

        public ActionResult Delete(int id)
        {
            var entity = EntityRepository.FindById(id);
            EntityRepository.Delete(entity);


            return RedirectToAction("Index");
        }
    }
}
