using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using System.Web.Mvc;
using Cmt6.Models;
using Microsoft.CSharp;

namespace Cmt6.Presentations.Mvc2.ModelBinders
{
    public class LambdaExpressionBinder<TEntity> : IModelBinder
        where TEntity : IContentEntity
    {
        public object BindModel(ControllerContext controllerContext,
                                ModelBindingContext bindingContext)
        {
            // Get the raw attempted value from the value provider 
            var key = bindingContext.ModelName;
            var value = bindingContext.ValueProvider.GetValue(key);
            if ((value != null) && !string.IsNullOrEmpty(value.AttemptedValue))
            {

                // Follow convention by stashing attempted value in ModelState 
                bindingContext.ModelState.SetModelValue(key, value);

                // Try to parse incoming data 
                var incomingString = ((string[])value.RawValue)[0];
                if (incomingString.Contains("=>"))
                {
                    return GetFromString(incomingString);
                }

                return null;
            }

            // No value was found in the request 
            return null;
        }

        private Func<TEntity, bool> GetFromString(string source)
        {
            Assembly a;
            using (var provider = new CSharpCodeProvider()) 
            {
                var assemblies = new List<string>();
                foreach (var x in AppDomain.CurrentDomain.GetAssemblies()) {
                    try {
                        assemblies.Add(x.Location);
                    }
                    catch (NotSupportedException) {
                        // Dynamic assemblies will throw, and in .net 3.5 there seems to be no way of finding out whether the assembly is dynamic before trying.
                    }
                }

                var r = provider.CompileAssemblyFromSource(new CompilerParameters(assemblies.ToArray()) { GenerateExecutable = false, GenerateInMemory = true }, source);
                if (r.Errors.HasErrors)
                    throw new Exception("Errors compiling expression: " + string.Join(Environment.NewLine, r.Errors.OfType<CompilerError>().Select(e => e.ErrorText).ToArray()));
                a = r.CompiledAssembly;
            }
            var o = a.CreateInstance("ExpressionContainer");
            var expression =  (Expression<Func<TEntity, bool>>) o.GetType().GetProperty("TheExpression").GetValue(o, null);
            return expression.Compile();
        }
    }
}