using System;
using System.Web.Mvc;

namespace Cmt6.Presentations.Mvc2.ModelBinders
{
    public class TypeBinder : IModelBinder
    {
        public object BindModel(ControllerContext controllerContext,
                                ModelBindingContext bindingContext)
        {
            // Get the raw attempted value from the value provider 
            var key = bindingContext.ModelName;
            var value = bindingContext.ValueProvider.GetValue(key);
            if ((value != null) && !string.IsNullOrEmpty(value.AttemptedValue))
            {

                // Follow convention by stashing attempted value in ModelState 
                bindingContext.ModelState.SetModelValue(key, value);

                // Try to parse incoming data 
                var incomingString = ((string[])value.RawValue)[0];
                if (incomingString.EndsWith("?"))
                {
                    //treat as nullable type
                    incomingString = string.Format("System.Nullable`1[{0}]",
                                                   incomingString.Substring(0, incomingString.Length - 1));
                }
                var type = Type.GetType(incomingString, false, true);
                if (type == null)
                    bindingContext.ModelState.AddModelError(key, "Can not find this type");

                return type;
            }

            // No value was found in the request 
            return null;
        } 
    }
}