using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Data.Linq.Mapping;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;
using Cmt6.Models;
using LinqToCodedom;
using LinqToCodedom.Extensions;
using LinqToCodedom.Generator;
using LinqToCodedom.Visitors;

namespace Cmt6.Repositories.LinqToSql
{
    public class ContentEntityWrapper<TEntity>
        where TEntity : IContentEntity
    {
        public IEntityRepository<IProperty<TEntity>> EntityPropertyRepository { get; set; }
        protected static IDictionary<Type, Type> WrappedTypesCache { get; private set; }

        static ContentEntityWrapper()
        {
            WrappedTypesCache = new Dictionary<Type, Type>();
        }

        //public Type Wrap(Type originalType)
        //{
        //    var asmName = typeof(TEntity).Name + DateTime.Now.Ticks;
        //    var asmBuilder = AppDomain.CurrentDomain.DefineDynamicAssembly(new AssemblyName(asmName), AssemblyBuilderAccess.Run);
        //    var modBuilder = asmBuilder.DefineDynamicModule(asmName);
        //    var typeBuilder = modBuilder.DefineType(asmName, TypeAttributes.Public, originalType);

        //    var tableAttributeCtorInfo = typeof(TableAttribute).GetConstructor(Type.EmptyTypes);
        //    var tableAttributeBuilder = new CustomAttributeBuilder(tableAttributeCtorInfo, new object[] { });
        //    typeBuilder.SetCustomAttribute(tableAttributeBuilder);

        //    var properties = EntityUtil.GetReadableProperties(originalType);
        //    foreach (var property in properties)
        //    {
        //        //todo read propery infomations from PropertyRepository
        //        ConstructorInfo columnAttributeCtorInfo;
        //        CustomAttributeBuilder columnAttributeBuilder;
        //        if (property.Name.Equals("Id", StringComparison.OrdinalIgnoreCase))
        //        {
        //            var isPrimaryKeyAttrPropertyInfo = typeof (ColumnAttribute).GetProperties()
        //                .First(p => p.Name.Equals("IsPrimaryKey", StringComparison.OrdinalIgnoreCase));
        //            columnAttributeCtorInfo = typeof(ColumnAttribute).GetConstructor(Type.EmptyTypes);
        //            columnAttributeBuilder = new CustomAttributeBuilder(columnAttributeCtorInfo, new object[] {},
        //                                                                new[] {isPrimaryKeyAttrPropertyInfo},
        //                                                                new object[] {true});
        //        }
        //        else
        //        {
        //            columnAttributeCtorInfo = typeof(ColumnAttribute).GetConstructor(Type.EmptyTypes);
        //            columnAttributeBuilder = new CustomAttributeBuilder(columnAttributeCtorInfo, new object[] { });
        //        }

        //        var propertyBuilder = typeBuilder.DefineProperty(property.Name, PropertyAttributes.HasDefault,
        //                                                         CallingConventions.HasThis, property.PropertyType,
        //                                                         new[] { property.PropertyType });
        //        propertyBuilder.SetCustomAttribute(columnAttributeBuilder);
        //    }

        //    return typeBuilder.CreateType();
        //}

        public Type Wrap(Type originalType)
        {
            if (WrappedTypesCache.ContainsKey(originalType))
                return WrappedTypesCache[originalType];
            var c = new CodeDomGenerator();
            var name = originalType.Name + DateTime.Now.Ticks;
            var interfaceType = typeof (TEntity);

            var asmNames = new List<AssemblyName>();
            asmNames.Add(originalType.Assembly.GetName());
            asmNames.Add((typeof(TEntity).Assembly.GetName()));

            //asms.Add(interfaceType.Assembly);
            //asmNames.Add(new AssemblyName("System.Data.Linq.dll"));
            //asmNames.Add(new AssemblyName("System.Data.dll"));
            //asmNames.Add(new AssemblyName("System.Core.dll"));
            //asmNames.Add(Assembly.LoadFrom("System.Data.dll"));
            //asmNames.Add(Assembly.LoadFrom("System.Core.dll"));
            //var baseTypeOfOriginalType = originalType.BaseType;
            //while (baseTypeOfOriginalType != typeof(object))
            //{
            //    asms.Add(baseTypeOfOriginalType.Assembly);
            //    baseTypeOfOriginalType = baseTypeOfOriginalType.BaseType;
            //}

            asmNames.AddRange(originalType.Assembly.GetReferencedAssemblies());
            asmNames.Remove(typeof (object).Assembly.GetName());

            foreach (var assembly in asmNames.Where(asmName => !asmName.Name.Equals(typeof(object).Assembly.GetName().Name)).Distinct())
            {
                c.AddReference(Assembly.Load(assembly).Location);
            }
            c.AddReference("System.Data.Linq.dll");
            c.AddReference("System.Data.dll");
            c.AddReference("System.Core.dll");
            //c.AddReference(originalType.Assembly.Location);
            //foreach (var assembly in originalType.Assembly.GetReferencedAssemblies())
            //{
            //    c.AddReference(Assembly.Load(assembly.Name).Location);
            //}

            var proxyType = c.AddNamespace(name)
                //.Imports(originalType.Namespace)
                //.Imports(interfaceType.Namespace)
                //.Imports("System.Data.Linq.Mapping")
                .AddClass(name)
                .Inherits(originalType)
                .Implements(interfaceType);
                //.AddAttribute(typeof (TableAttribute));
            var tableName = interfaceType.Name.Substring(1);
            var tableAtttribute = new CodeAttributeDeclaration(new CodeTypeReference(typeof (TableAttribute)),
                                                               new CodeAttributeArgument("Name",
                                                                                         new CodeExpressionVisitor(
                                                                                             new VisitorContext()).Visit
                                                                                             ((Expression<Func<string>>)
                                                                                              (() => tableName))));
            proxyType.AddAttribute(tableAtttribute);
            proxyType.AddField(originalType, "entity");
            proxyType.AddCtor(Emit.assignField("entity", () => CodeDom.@new(originalType)));
            foreach (var pi in EntityUtil.GetReadableProperties(originalType))
            {
                var propertyInfo = pi;
                if (proxyType.Members.Cast<CodeTypeMember>().Any(m => m.Name.Equals(propertyInfo.Name)))
                {
                    continue;
                }
                var property = proxyType.AddProperty(propertyInfo.PropertyType, MemberAttributes.Override | MemberAttributes.Public,
                                                     propertyInfo.Name
                                                     , new[]
                                                           {
                                                               Emit.declareVar("e", () => CodeDom.@this.Field("entity"))
                                                               ,
                                                               Emit.@return((Var e) => e.Property(propertyInfo.Name))
                                                           }
                                                     , new[]
                                                           {
                                                               Emit.assignProperty(
                                                                   new CodeExpressionVisitor(new VisitorContext()).Visit
                                                                       ((Expression<Func<object>>)
                                                                        (() => CodeDom.@this.Field("entity"))),
                                                                   propertyInfo.Name, () => CodeDom.VarRef("value"))
                                                           }
                    );
                property.AddAttribute(typeof (ColumnAttribute));
            }

            var source = c.GenerateCode(CodeDomGenerator.Language.CSharp);
            var ass = c.Compile(CodeDomGenerator.Language.CSharp);
            var rv = ass.GetType(string.Format("{0}.{1}", name, name));
            WrappedTypesCache.Add(originalType, rv);
            return rv;
        }
    }
}