using System;
using Cmt6.Models;

namespace Cmt6.Repositories.LinqToSql
{
    public abstract class PropertyEntityRepositoryBase<TEntity> : MockEntityRepositoryBase<IProperty<TEntity>>, IPropertyRepository<TEntity>
        where TEntity : IContentEntity
    {
        protected Action<IProperty<TEntity>> m_addedAction;
        protected Action<IProperty<TEntity>> m_updatedAction;
        protected Action<IProperty<TEntity>> m_deletedAction;

        public event Action<IProperty<TEntity>> Added
        {
            add { m_addedAction += value; }
            remove { m_addedAction -= value; }
        }
        public event Action<IProperty<TEntity>> Updated
        {
            add { m_updatedAction += value; }
            remove { m_updatedAction -= value; }
        }
        public event Action<IProperty<TEntity>> Deleted
        {
            add { m_deletedAction += value; }
            remove { m_deletedAction -= value; }
        }

        public override void Add(IProperty<TEntity> entity)
        {
            base.Add(entity);
            if (m_addedAction != null)
            {
                m_addedAction.Invoke(entity);
            }
        }

        public override void Update(IProperty<TEntity> entity)
        {
            base.Update(entity);
            if (m_updatedAction != null)
            {
                m_updatedAction.Invoke(entity);
            }
        }

        public override void Delete(IProperty<TEntity> entity)
        {
            base.Delete(entity);
            if (m_deletedAction != null)
            {
                m_deletedAction.Invoke(entity);
            }
        }
    }
}