using System.Collections.Generic;
using System.Linq;
using Cmt6.Models;

namespace Cmt6.Repositories.Mock
{
    public abstract class MockEntityRepositoryBase<TEntity> : EntityRepositoryBase<TEntity>
        where TEntity : class, IEntity
    {
        protected IList<TEntity> Entities { get; set; }

        protected MockEntityRepositoryBase()
        {
            Entities = new List<TEntity>();
        }

        public override IEnumerable<TEntity> FindAll()
        {
            return Entities;
        }

        public override void Add(TEntity entity)
        {
            Entities.Add(entity);
        }

        public override void Delete(TEntity entity)
        {
            var entityToRemove = Entities.FirstOrDefault(e => e.Id == entity.Id);
            if (entityToRemove.Equals(default(TEntity)))
                return;
            Entities.Remove(entityToRemove);
        }

        public override void Update(TEntity entity)
        {
            var entityToUpdate = Entities.FirstOrDefault(e => e.Id == entity.Id);
            if (entityToUpdate == null)
                return;
            Entities.Remove(entityToUpdate);
            Entities.Add(entity);
        }

        public override void AddOrUpdate(TEntity entity)
        {
            var entityToUpdate = Entities.FirstOrDefault(e => e.Id == entity.Id);
            if (entityToUpdate != null)
            {
                Entities.Remove(entityToUpdate);
            }
            Entities.Add(entity);
        }

        protected int GenerateNextId()
        {
            var currentMaxId = Entities.Max(entity => entity.Id);
            return currentMaxId + 1;
        }
    }
}