// Type: LinqToCodedom.Extensions.CodeTypeMemberExtensions
// Assembly: LinqToCodedom, Version=1.0.0.0, Culture=neutral, PublicKeyToken=8b32a020163181f7
// Assembly location: D:\work\ctx\CMT45\Reshearch\DynamicComponent\lib\LinqToCodedom.dll

using System;
using System.CodeDom;
using System.Linq.Expressions;

namespace LinqToCodedom.Extensions
{
    public static class CodeTypeMemberExtensions
    {
        public static CodeTypeDeclaration AddFields(this CodeTypeMember member, params CodeMemberField[] field);

        public static CodeMemberField AddField(this CodeTypeMember member, Type fieldType, MemberAttributes ma,
                                               string name);

        public static CodeMemberField AddField(this CodeTypeMember member, Type fieldType, string name);

        public static CodeMemberField AddField<T>(this CodeTypeMember member, Type fieldType, string name,
                                                  Expression<Func<T>> exp);

        public static CodeTypeDeclaration AddProperty(this CodeTypeMember member, CodeMemberProperty property);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, string propertyType,
                                                     MemberAttributes ma, string name, string fieldName);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, Type propertyType, MemberAttributes ma,
                                                     string name, string fieldName);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, string propertyType,
                                                     MemberAttributes ma, string name);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, Type propertyType, MemberAttributes ma,
                                                     string name);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, CodeTypeReference propertyType,
                                                     MemberAttributes ma, string name);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, Type propertyType, MemberAttributes ma,
                                                     string name, CodeStatement[] getStatements,
                                                     params CodeStatement[] setStatements);

        public static CodeMemberProperty AddProperty(this CodeTypeMember member, string propertyType,
                                                     MemberAttributes ma, string name, CodeStatement[] getStatements,
                                                     params CodeStatement[] setStatements);

        public static CodeMemberProperty AddGetProperty(this CodeTypeMember member, Type propertyType,
                                                        MemberAttributes ma, string name,
                                                        params CodeStatement[] statements);

        public static CodeMemberProperty AddGetProperty(this CodeTypeMember member, string propertyType,
                                                        MemberAttributes ma, string name,
                                                        params CodeStatement[] statements);

        public static CodeMemberProperty AddGetProperty(this CodeTypeMember member, Type propertyType,
                                                        MemberAttributes ma, string name, string fieldName);

        public static CodeMemberProperty AddGetProperty(this CodeTypeMember member, string propertyType,
                                                        MemberAttributes ma, string name);

        public static CodeMemberProperty Property(this CodeTypeMember member, Type propertyType, MemberAttributes ma,
                                                  string name, CodeStatement[] getStatements,
                                                  params CodeStatement[] setStatements);

        public static T Comment<T>(this T member, params CodeCommentStatement[] comments) where T : CodeTypeMember;
        public static T Comment<T>(this T member, params string[] comments) where T : CodeTypeMember;
        public static T Document<T>(this T member, params string[] comments) where T : CodeTypeMember;

        public static T AddAttribute<T>(this T member, params CodeAttributeDeclaration[] attributes)
            where T : CodeTypeMember;

        public static T AddAttribute<T>(this T member, params Type[] attributes) where T : CodeTypeMember;
        public static T AddAttribute<T>(this T member, params string[] attributes) where T : CodeTypeMember;
        public static CodeTypeDeclaration AddClass(this CodeTypeMember member, string className);
        public static T StartDirective<T>(this T member, CodeDirective directive) where T : CodeTypeMember;
        public static T StartDirective<T>(this T member, string title) where T : CodeTypeMember;
        public static T EndDirective<T>(this T member, CodeDirective directive) where T : CodeTypeMember;
        public static T EndDirective<T>(this T member) where T : CodeTypeMember;
        public static T WrapRegion<T>(this T member, string title) where T : CodeTypeMember;
        public static T LineMark<T>(this T member, string file, int line) where T : CodeTypeMember;
        public static CodeTypeDeclaration AddEvents(this CodeTypeMember member, params CodeMemberEvent[] @event);

        public static CodeMemberEvent AddEvent(this CodeTypeMember member, string delegateType, MemberAttributes ma,
                                               string name);

        public static CodeMemberEvent AddEvent(this CodeTypeMember member, Type delegateType, MemberAttributes ma,
                                               string name);

        public static CodeTypeDeclaration AddMethod(this CodeTypeMember member, CodeMemberMethod methodBody);

        public static CodeMemberMethod AddMethod<T>(this CodeTypeMember member, MemberAttributes ma, Type returnType,
                                                    Expression<Func<T, string>> paramsAndName,
                                                    params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod<T, T2>(this CodeTypeMember member, MemberAttributes ma, Type returnType,
                                                        Expression<Func<T, T2, string>> paramsAndName,
                                                        params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod<T>(this CodeTypeMember member, MemberAttributes ma, string returnType,
                                                    Expression<Func<T, string>> paramsAndName,
                                                    params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod(this CodeTypeMember member, MemberAttributes ma, string returnType,
                                                 Expression<Func<string>> paramsAndName,
                                                 params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod(this CodeTypeMember member, MemberAttributes ma, Type returnType,
                                                 Expression<Func<string>> paramsAndName,
                                                 params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod<T>(this CodeTypeMember member, MemberAttributes ma,
                                                    Expression<Func<T, string>> paramsAndName,
                                                    params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod<T, T2>(this CodeTypeMember member, MemberAttributes ma,
                                                        Expression<Func<T, T2, string>> paramsAndName,
                                                        params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod<T, T2, T3>(this CodeTypeMember member, MemberAttributes ma,
                                                            Expression<Func<T, T2, T3, string>> paramsAndName,
                                                            params CodeStatement[] statements);

        public static CodeMemberMethod AddMethod(this CodeTypeMember member, MemberAttributes ma,
                                                 Expression<Func<string>> paramsAndName,
                                                 params CodeStatement[] statements);

        public static CodeTypeMember AddCtor(this CodeTypeMember member, CodeConstructor ctor);

        public static CodeConstructor AddCtor<T>(this CodeTypeMember member,
                                                 Expression<Func<T, MemberAttributes>> paramsAndAccessLevel,
                                                 params CodeStatement[] statements);

        public static CodeConstructor AddCtor(this CodeTypeMember member,
                                              Expression<Func<MemberAttributes>> paramsAndAccessLevel,
                                              params CodeStatement[] statements);
    }
}
